#include <stdio.h>   /* Standard input/output definitions */
#include <string.h>  /* String function definitions */
#include <unistd.h>  /* UNIX standard function definitions */
#include <fcntl.h>   /* File control definitions */
#include <errno.h>   /* Error number definitions */
#include <termios.h> /* POSIX terminal control definitions */
#include <math.h>
#include <time.h>

#include "i18n.h"
#include "activylcd.h"
#include <vdr/tools.h>
#include <vdr/config.h>

#include <vdr/plugin.h>
#include <vdr/status.h>
#include <vdr/channels.h>

extern int PowerButton;
extern int WaitTime;
extern int RecordLED;
extern int LEDBrightness;
extern int LEDBrightnessOnExit;

aLCD::aLCD()
{
  wait = 0; // waittimer for status messages
  refresh = 0;
  stop = false;
  title=false;
  memset(line0,' ',20);
  memset(line1,' ',20);
  memset(oldline0,' ',20);
  memset(oldline1,' ',20);
  memset(chline0,' ',20);
  memset(chline1,' ',20);
  memset(volline1,' ',20);
#ifdef ADEBUG
  open_port();
#endif
  init_display();  
}

aLCD::~aLCD()
{
  stop = true;
  while (running) usleep(100000);
  clear_text();
  powerbutton(1);
  SetLEDBrightness(LEDBrightnessOnExit);
  close_port();
}

#ifdef ADEBUG
FILE* aLCD::open_port(void)
{
  fd = fopen("/tmp/test.dat","w");
  if (fd) {
    isyslog("open_port: open file /tmp/test.dat");
  }
  return (fd);
}
#else
int aLCD::open_port(void)
{
  fd = open(LCD_PORT, O_RDWR | O_NOCTTY | O_NDELAY);
  if (fd == -1) {
    isyslog("alcd: Unable to open %s", LCD_PORT);
  } else {
    fcntl(fd, F_SETFL, 0);
  }
  setup_port();
  return (fd);
}
#endif

void aLCD::close_port()
{
#ifdef ADEBUG
  fclose(fd);
#else
  close(fd);
#endif
}

void aLCD::init_display()
{
  clear_text();
  if(PowerButton)
  {
    powerbutton(1);
  } else {
    powerbutton(0);
  }
  SetLEDBrightness(LEDBrightness);
  refresh = 1;
}

void aLCD::setup_port()
{
#ifndef ADEBUG
  struct termios options;
  // Get the current options for the port...
  tcgetattr(fd, &options);
  // Set the baud rates to 38400...
  cfsetispeed(&options, B38400);
  cfsetospeed(&options, B38400);
  // Enable the receiver and set local mode...
  options.c_cflag |= (CLOCAL);
  options.c_cflag &= ~PARENB;
  options.c_cflag &= ~CSTOPB;
  options.c_cflag &= ~CSIZE;
  options.c_cflag |= CS8;
  options.c_oflag &= ~OPOST;
  // Set the new options for the port...
  tcsetattr(fd, TCSANOW, &options);
#endif
}

void aLCD::print_line(int line, char text[21])
{
  switch (line) {
    case 0:
#ifdef ADEBUG
      fprintf(fd, "%s%s%c", LCD_WRITE_TEXT_LINE0, text,'\x00');
      isyslog("refresh line0 '%s', recordled %d waittime %d", text, RecordLED, WaitTime);
#else
      open_port();
      snprintf(output,23, "%s%s%c", LCD_WRITE_TEXT_LINE0, text, '\x00');
      write(fd, &output,23 );
      close_port();
//      isyslog("refresh line0 '%s'", text);
#endif
      break;
    case 1:
#ifdef ADEBUG
      fprintf(fd, "%s%s%c", LCD_WRITE_TEXT_LINE1, text,'\x00');
      isyslog("refresh line1 '%s', recordled %d waittime %d", text, RecordLED, WaitTime);
#else
      open_port();
      snprintf(output,23, "%s%s%c", LCD_WRITE_TEXT_LINE1, text,'\x00');
      write(fd, &output,23 );
      close_port();
//      isyslog("refresh line1 '%s'", text);
#endif
      break;
  }
#ifdef ADEBUG
  fflush(fd);
#endif
  
}



void aLCD::clear_text() // Don't use in running thread, instead assign line1 and line2 !!!
{
#ifdef ADEBUG
  fprintf(fd, "%s%c", LCD_CLEAR, '\x00');
#else
  open_port();
  snprintf(output,2, "%s%c", LCD_CLEAR, '\x00');
  write(fd, &output,2);
  close_port();
#endif
}

void aLCD::powerbutton(bool status) // switch powerbutton on
{
  switch (status) {
    case 0:
#ifdef ADEBUG
      fprintf(fd, "%s", POWERBUTTON_SCANCODE_OFF);
      fprintf(fd, "%s", POWERBUTTON_OFF);
      isyslog("powerbutton off");
#else
      open_port();
      snprintf(output,2, "%s", POWERBUTTON_SCANCODE_OFF);
      write(fd, &output,2);
      close_port();
      open_port();
      snprintf(output,2, "%s", POWERBUTTON_OFF);
      write(fd, &output,2);
      close_port();
#endif
      break;
    case 1:
#ifdef ADEBUG
      fprintf(fd, "%s", POWERBUTTON_SCANCODE_ON);
      fprintf(fd, "%s", POWERBUTTON_ON);
      isyslog("powerbutton on");
#else
      open_port();
      snprintf(output,2, "%s", POWERBUTTON_SCANCODE_ON);
      write(fd, &output,2);
      close_port();
      open_port();
      snprintf(output,2, "%s", POWERBUTTON_ON);
      write(fd, &output,2);
      close_port();
#endif
      break;
  }
}

void aLCD::SetLEDBrightness(const int brightness)
{
  int brightness_temp = 0;
  switch(brightness) { 
    case 0: // superbright
      brightness_temp = 0;
      break;
    case 1: // bright
      brightness_temp = 50;
      break;
    case 2: // semibright
      brightness_temp = 100;
      break;
    case 3: // dark
      brightness_temp = 150;
      break;
    case 4: // off
      brightness_temp = 255;
      break;
  }

#ifdef ADEBUG
  fprintf(fd, "%s%c", LCD_LEDBRIGHTNESS, brightness_temp);
  isyslog("brightness set to %d", brightness_temp);
#else
  open_port();
  snprintf(output,2, "%s%c", LCD_LEDBRIGHTNESS, brightness_temp);
  write(fd, &output,2);
  close_port();
#endif

}


void aLCD::SetChannelName (const char *Name)
{
  snprintf(chline0,21,"%s",Name);
  memset(chline1,' ',20);
}

void aLCD::SetProgramme (const char *PresentTitle)
{
  snprintf(chline1,21,"%s",PresentTitle);
}


void aLCD::SetChannel (int  ChannelNumber)
{
  snprintf(chline0,5,"%-4d",ChannelNumber);
}

void aLCD::SetRecording(int card, const char *Name)
{
  if (Name != NULL) {
    switch (RecordLED) {
      case 1:
#ifdef ADEBUG
        fprintf(fd, "%s", MESSAGELED_BLINK_ON);
        fprintf(fd, "%s", BASICLED_OFF);
#else
        open_port();
	snprintf(output,2, "%s", MESSAGELED_BLINK_ON);
	write(fd, &output,2);
        close_port();
        open_port();
	snprintf(output,2, "%s", BASICLED_OFF);
	write(fd, &output,2);
        close_port();
#endif
        break;
      case 2:
#ifdef ADEBUG
        fprintf(fd, "%s", BASICLED_ON);
	fprintf(fd, "%s", MESSAGELED_OFF);
#else
        open_port();
	snprintf(output,2, "%s",BASICLED_ON);
	write(fd, &output,2);
        close_port();
        open_port();
	snprintf(output,2, "%s",MESSAGELED_OFF);
	write(fd, &output,2);
        close_port();
#endif
        break;

    };
    snprintf(RecordName,41,"%s", Name);
    snprintf(recline0,21,tr("Recording started:"));
    snprintf(recline1,21,"%-20s", RecordName);
  } else {
#ifdef ADEBUG
    fprintf(fd, "%s", MESSAGELED_OFF);
    fprintf(fd, "%s", BASICLED_OFF); 
#else
    open_port();
    snprintf(output,2, "%s", MESSAGELED_OFF);
    write(fd, &output,2);
    close_port();
    open_port();
    snprintf(output,2, "%s", BASICLED_OFF);
    write(fd, &output,2);
    close_port();
    snprintf(recline0,21,tr("Recording stopped:"));
    snprintf(recline1,100,"%s", RecordName);
#endif
  };
  lcdmodus = 1;
}

void aLCD::ReplayControl(cControl *Control, const char *Name)
{
  if (Name == NULL) {
    lcdmodus = 0;
  } else {
    lcdmodus = 3;
    PlayControl = Control;
    snprintf(chline0,21,"%-20s",Name);
    snprintf(chline1,21,"                  > ");
  }
}

void aLCD::SetVolume (int volume, bool muted)
{
  percent = (int) lround(volume/14); // 1 -> 18
#ifdef ADEBUG
  isyslog("SetVolume Vol %d mute %d -- percent %d", volume, muted, percent);
#endif
  if(muted)
  {
    snprintf(volline1,21,"[    -- MUTE --    ]");
  } else {
    SetVolumeBar(percent);
  }
  oldmodus=lcdmodus;
  lcdmodus=2;
}

void aLCD::SetVolumeBar(int percent)
{
  int temp;
  char vol_temp[18];
  for (temp=1;temp<percent+1;temp++) {
    strcat(vol_temp, "#");
  }
  snprintf(volline1,21,"[%-18s]", vol_temp);
}

void aLCD::SetProgressBar(int percent)
{
  int temp;
  char prog_temp[15];
    for (temp=1;temp<percent+1;temp++) {
    strcat(prog_temp, "#");
  }
  snprintf(chline1,21,"[%-15s]", prog_temp);
}

void aLCD::Action(void)
{
  running=true;
  while(!stop){
    gettimeofday(&now,NULL);
    struct tm uhr;
    time_t val;
    switch (lcdmodus) { 
      case 0: // channeldisplay
	val = time(NULL);
	localtime_r(&val,&uhr);
        chline0[14]='\0';
        snprintf(line0,21,"%-14s %02d:%02d", chline0, uhr.tm_hour, uhr.tm_min);
        snprintf(line1,21,"%-20s", chline1);
	wait = 0;
        break;
      case 1: // display recording
        snprintf(line0,21,"%-20s", recline0);
        snprintf(line1,21,"%-20s", recline1);
        wait++;
        if(wait>(WaitTime*10)) {
          snprintf(line1,21,"%-20s", chline1);
          lcdmodus = 0; // switch back to time / chan display
          wait = 0;
        };
	break;
      case 2: // volume display
	val = time(NULL);
	localtime_r(&val,&uhr);
        snprintf(line0,21,"%-14s %02d:%02d", chline0, uhr.tm_hour, uhr.tm_min);
        snprintf(line1,21,"%-20s", volline1);
        wait++;
        if(wait>(WaitTime*10)) {
          snprintf(line1,21,"%-20s", volline1);
          lcdmodus = 0; // switch back to time / chan display
          wait = 0;
        };
	break;
      case 3: // replay display
        char PlaySymbol[3];
        char IndexTime[17];
        int Current, Total, Speed;
        bool Play, Forward;
        if (PlayControl->GetReplayMode(Play,Forward,Speed)) {
          if (Play && Speed == -1)     snprintf(PlaySymbol,2,"%2s","> "); // Normal Replay
          if (!Play && Speed == -1)    snprintf(PlaySymbol,2,"%2s","||"); // Pause
          if (Forward && Speed != -1)  snprintf(PlaySymbol,2,"%2s",">>"); // Fast Forward
          if (!Forward && Speed != -1) snprintf(PlaySymbol,2,"%2s","<<"); // Fast Backward
        }
        if (PlayControl->GetIndex(Current,Total)) {
	  snprintf(IndexTime,18,"%-17s",IndexToHMSF(Current,false));
          snprintf(chline1,21,"%-17s %-2s",IndexTime, PlaySymbol);
        }
        snprintf(line0,21,"%-20s", chline0);
        snprintf(line1,21,"%-20s", chline1);
    }
    	
    if ((strcmp(line0,oldline0)!=0 && refresh) || (strcmp(line1,oldline1)!=0 && refresh)) {
      memcpy(oldline0,line0,21);
      memcpy(oldline1,line1,21); 
      print_line(0,line0);
      print_line(1,line1);
    }

    usleep(100000-(now.tv_usec%100000));
  }
  running=false;
}

